@@ -29,6 +29,7 @@
 
 #include "pango-bidi-type.h"
 #include "pango-utils.h"
+#include "pango-utils-private.h"
 
 /**
  * pango_bidi_type_for_unichar:
@@ -96,30 +97,50 @@ pango_bidi_type_for_unichar (gunichar ch)
  *
  * Return the bidirectional embedding levels of the input paragraph.
  *
- * The bidirectional embedding levels are defined by the Unicode Bidirectional
- * Algorithm available at:
- *
- *   http://www.unicode.org/reports/tr9/
+ * The bidirectional embedding levels are defined by the [Unicode Bidirectional
+ * Algorithm](http://www.unicode.org/reports/tr9/).
  *
  * If the input base direction is a weak direction, the direction of the
  * characters in the text will determine the final resolved direction.
  *
  * Return value: a newly allocated array of embedding levels, one item per
- *   character (not byte), that should be freed using g_free().
+ *   character (not byte), that should be freed using [func@GLib.free].
  *
  * Since: 1.4
  */
 guint8 *
 pango_log2vis_get_embedding_levels (const gchar    *text,
-				    int             length,
-				    PangoDirection *pbase_dir)
+                                    int             length,
+                                    PangoDirection *pbase_dir)
 {
-  glong n_chars, i;
-  guint8 *embedding_levels_list;
+  unsigned int n_chars;
+  guint8 *embedding_levels;
+
+  if (length < 0)
+    length = strlen (text);
+
+  n_chars = g_utf8_strlen (text, length);
+  embedding_levels = g_new (guint8, n_chars);
+
+  pango_log2vis_fill_embedding_levels (text, length, n_chars, embedding_levels, pbase_dir);
+
+  return embedding_levels;
+}
+
+void
+pango_log2vis_fill_embedding_levels (const gchar    *text,
+                                    int             length,
+                                    unsigned int    n_chars,
+                                    guint8         *embedding_levels_list,
+                                    PangoDirection *pbase_dir)
+{
+  glong i;
   const gchar *p;
   FriBidiParType fribidi_base_dir;
   FriBidiCharType *bidi_types;
+  FriBidiCharType bidi_types_[64];
   FriBidiBracketType *bracket_types;
+  FriBidiBracketType bracket_types_[64];
   FriBidiLevel max_level;
   FriBidiCharType ored_types = 0;
   FriBidiCharType anded_strongs = FRIBIDI_TYPE_RLE;
@@ -147,14 +168,16 @@ pango_log2vis_get_embedding_levels (const gchar    *text,
       break;
     }
 
-  if (length < 0)
-    length = strlen (text);
-
-  n_chars = g_utf8_strlen (text, length);
-
-  bidi_types = g_new (FriBidiCharType, n_chars);
-  bracket_types = g_new (FriBidiBracketType, n_chars);
-  embedding_levels_list = g_new (guint8, n_chars);
+  if (n_chars < 64)
+    {
+      bidi_types = bidi_types_;
+      bracket_types = bracket_types_;
+    }
+  else
+    {
+      bidi_types = g_new (FriBidiCharType, n_chars);
+      bracket_types = g_new (FriBidiBracketType, n_chars);
+    }
 
   for (i = 0, p = text; p < text + length; p = g_utf8_next_char(p), i++)
     {
@@ -234,12 +257,13 @@ pango_log2vis_get_embedding_levels (const gchar    *text,
     }
 
 resolved:
-  g_free (bidi_types);
-  g_free (bracket_types);
+  if (n_chars >= 64)
+    {
+      g_free (bidi_types);
+      g_free (bracket_types);
+    }
 
   *pbase_dir = (fribidi_base_dir == FRIBIDI_PAR_LTR) ?  PANGO_DIRECTION_LTR : PANGO_DIRECTION_RTL;
-
-  return embedding_levels_list;
 }
 
 /**
@@ -248,13 +272,13 @@ resolved:
  *
  * Determines the inherent direction of a character.
  *
- * The inherent direction is either %PANGO_DIRECTION_LTR, %PANGO_DIRECTION_RTL,
- * or %PANGO_DIRECTION_NEUTRAL.
+ * The inherent direction is either `PANGO_DIRECTION_LTR`, `PANGO_DIRECTION_RTL`,
+ * or `PANGO_DIRECTION_NEUTRAL`.
  *
  * This function is useful to categorize characters into left-to-right
  * letters, right-to-left letters, and everything else. If full Unicode
- * bidirectional type of a character is needed,
- * [func@Pango.BidiType.for_unichar] can be used instead.
+ * bidirectional type of a character is needed, [func@Pango.BidiType.for_unichar]
+ * can be used instead.
  *
  * Return value: the direction of the character.
  */
@@ -288,8 +312,8 @@ pango_unichar_direction (gunichar ch)
  * Return value: %TRUE if @ch has a mirrored character and @mirrored_ch is
  * filled in, %FALSE otherwise
  *
- * Deprecated: 1.30: Use g_unichar_get_mirror_char() instead; the docs for that function
- * provide full details.
+ * Deprecated: 1.30: Use [func@GLib.unichar_get_mirror_char] instead;
+ *   the docs for that function provide full details.
  */
 gboolean
 pango_get_mirror_char (gunichar  ch,
