@@ -27,6 +27,7 @@
 #include "pango-fontset-simple.h"
 #include "pango-impl-utils.h"
 #include <stdlib.h>
+#include <math.h>
 
 static PangoFontset *pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
                                                        PangoContext               *context,
@@ -39,22 +40,91 @@ static PangoFontFamily *pango_font_map_real_get_family (PangoFontMap *fontmap,
 
 static void pango_font_map_real_changed (PangoFontMap *fontmap);
 
+static PangoFont *pango_font_map_real_reload_font (PangoFontMap *fontmap,
+                                                   PangoFont    *font,
+                                                   double        scale,
+                                                   PangoContext *context,
+                                                   const char   *variations);
+
+static guint pango_font_map_get_n_items (GListModel *list);
+
 static void pango_font_map_list_model_init (GListModelInterface *iface);
 
 typedef struct {
   guint n_families;
 } PangoFontMapPrivate;
 
+enum
+{
+  PROP_0,
+  PROP_ITEM_TYPE,
+  PROP_N_ITEMS,
+  N_PROPERTIES
+};
+
+static GParamSpec *properties[N_PROPERTIES] = { NULL, };
+
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PangoFontMap, pango_font_map, G_TYPE_OBJECT,
                                   G_ADD_PRIVATE (PangoFontMap)
+                                  g_type_add_class_private (g_define_type_id, sizeof (PangoFontMapClassPrivate));
                                   G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, pango_font_map_list_model_init))
 
+static void
+pango_font_map_get_property (GObject    *object,
+                             guint       property_id,
+                             GValue     *value,
+                             GParamSpec *pspec)
+{
+  switch (property_id)
+    {
+    case PROP_ITEM_TYPE:
+      g_value_set_gtype (value, PANGO_TYPE_FONT_FAMILY);
+      break;
+
+    case PROP_N_ITEMS:
+      g_value_set_uint (value, pango_font_map_get_n_items (G_LIST_MODEL (object)));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
 static void
 pango_font_map_class_init (PangoFontMapClass *class)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+  PangoFontMapClassPrivate *pclass;
+
+  object_class->get_property = pango_font_map_get_property;
+
   class->load_fontset = pango_font_map_real_load_fontset;
   class->get_family = pango_font_map_real_get_family;
   class->changed = pango_font_map_real_changed;
+
+  pclass = g_type_class_get_private ((GTypeClass *) class, PANGO_TYPE_FONT_MAP);
+
+  pclass->reload_font = pango_font_map_real_reload_font;
+
+  /**
+   * PangoFontMap:item-type:
+   *
+   * The type of items contained in this list.
+   */
+  properties[PROP_ITEM_TYPE] =
+    g_param_spec_gtype ("item-type", "", "", G_TYPE_OBJECT,
+                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoFontMap:n-items:
+   *
+   * The number of items contained in this list.
+   */
+  properties[PROP_N_ITEMS] =
+    g_param_spec_uint ("n-items", "", "", 0, G_MAXUINT, 0,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 }
 
 static void
@@ -355,6 +425,8 @@ pango_font_map_real_changed (PangoFontMap *fontmap)
   added = g_list_model_get_n_items (G_LIST_MODEL (fontmap));
 
   g_list_model_items_changed (G_LIST_MODEL (fontmap), 0, removed, added);
+  if (removed != added)
+    g_object_notify_by_pspec (G_OBJECT (fontmap), properties[PROP_N_ITEMS]);
 }
 
 /**
@@ -427,6 +499,78 @@ pango_font_map_get_family (PangoFontMap *fontmap,
   return PANGO_FONT_MAP_GET_CLASS (fontmap)->get_family (fontmap, name);
 }
 
+static PangoFont *
+pango_font_map_real_reload_font (PangoFontMap *fontmap,
+                                 PangoFont    *font,
+                                 double        scale,
+                                 PangoContext *context,
+                                 const char   *variations)
+{
+  PangoFontDescription *desc;
+  PangoContext *freeme = NULL;
+  PangoFont *scaled;
+
+  desc = pango_font_describe_with_absolute_size (font);
+
+  if (scale != 1.0)
+    {
+      int size = pango_font_description_get_size (desc);
+
+      pango_font_description_set_absolute_size (desc, size * scale);
+    }
+
+  if (!context)
+    freeme = context = pango_font_map_create_context (fontmap);
+
+  if (variations)
+    pango_font_description_set_variations_static (desc, variations);
+
+  scaled = pango_font_map_load_font (fontmap, context, desc);
+
+  g_clear_object (&freeme);
+
+  pango_font_description_free (desc);
+
+  return scaled;
+}
+
+/**
+ * pango_font_map_reload_font:
+ * @fontmap: a `PangoFontMap`
+ * @font: a font in @fontmap
+ * @scale: the scale factor to apply
+ * @context: (nullable): a `PangoContext`
+ * @variations: (nullable): font variations to use
+ *
+ * Returns a new font that is like @font, except that its size
+ * is multiplied by @scale, its backend-dependent configuration
+ * (e.g. cairo font options) is replaced by the one in @context,
+ * and its variations are replaced by @variations.
+ *
+ * Returns: (transfer full): the modified font
+ *
+ * Since: 1.52
+ */
+PangoFont *
+pango_font_map_reload_font (PangoFontMap *fontmap,
+                            PangoFont    *font,
+                            double        scale,
+                            PangoContext *context,
+                            const char   *variations)
+{
+  PangoFontMapClassPrivate *pclass;
+
+  g_return_val_if_fail (PANGO_IS_FONT (font), NULL);
+  g_return_val_if_fail (fontmap == pango_font_get_font_map (font), NULL);
+  g_return_val_if_fail (scale > 0, NULL);
+  g_return_val_if_fail (context == NULL || PANGO_IS_CONTEXT (context), NULL);
+
+  pclass = g_type_class_get_private ((GTypeClass *) PANGO_FONT_MAP_GET_CLASS (fontmap),
+                                     PANGO_TYPE_FONT_MAP);
+
+  return pclass->reload_font (fontmap, font, scale, context, variations);
+}
+
 static GType
 pango_font_map_get_item_type (GListModel *list)
 {
