@@ -24,6 +24,7 @@
 #include "pango-font-private.h"
 #include "pangofc-font-private.h"
 #include "pangofc-fontmap.h"
+#include "pangofc-fontmap-private.h"
 #include "pangofc-private.h"
 #include "pango-layout.h"
 #include "pango-impl-utils.h"
@@ -78,6 +79,7 @@ static void                 _pango_fc_font_get_scale_factors (PangoFont     *fon
 static void                 pango_fc_font_get_matrix      (PangoFont        *font,
                                                            PangoMatrix      *matrix);
 static int                  pango_fc_font_get_absolute_size (PangoFont        *font);
+static PangoVariant         pango_fc_font_get_variant       (PangoFont        *font);
 
 #define PANGO_FC_FONT_LOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->lock_face (font))
 #define PANGO_FC_FONT_UNLOCK_FACE(font)	(PANGO_FC_FONT_GET_CLASS (font)->unlock_face (font))
@@ -114,6 +116,7 @@ pango_fc_font_class_init (PangoFcFontClass *class)
   pclass->get_scale_factors = _pango_fc_font_get_scale_factors;
   pclass->get_matrix = pango_fc_font_get_matrix;
   pclass->get_absolute_size = pango_fc_font_get_absolute_size;
+  pclass->get_variant = pango_fc_font_get_variant;
 
   /**
    * PangoFcFont:pattern:
@@ -159,21 +162,20 @@ pango_fc_font_finalize (GObject *object)
 {
   PangoFcFont *fcfont = PANGO_FC_FONT (object);
   PangoFcFontPrivate *priv = fcfont->priv;
-  PangoFcFontMap *fontmap;
+  PangoFontMap *fontmap;
 
   g_slist_foreach (fcfont->metrics_by_lang, (GFunc)free_metrics_info, NULL);
   g_slist_free (fcfont->metrics_by_lang);
 
-  fontmap = g_weak_ref_get ((GWeakRef *) &fcfont->fontmap);
+  fontmap = fcfont->fontmap;
   if (fontmap)
     {
-      _pango_fc_font_map_remove (PANGO_FC_FONT_MAP (fcfont->fontmap), fcfont);
-      g_weak_ref_clear ((GWeakRef *) &fcfont->fontmap);
-      g_object_unref (fontmap);
+      g_object_remove_weak_pointer (G_OBJECT (fontmap), (gpointer *) &fcfont->fontmap);
+      _pango_fc_font_map_remove (PANGO_FC_FONT_MAP (fontmap), fcfont);
     }
 
-  FcPatternDestroy (fcfont->font_pattern);
   pango_font_description_free (fcfont->description);
+  FcPatternDestroy (fcfont->font_pattern);
 
   if (priv->decoder)
     _pango_fc_font_set_decoder (fcfont, NULL);
@@ -209,9 +211,9 @@ pattern_is_transformed (FcPattern *pattern)
 
 static void
 pango_fc_font_set_property (GObject       *object,
-			    guint          prop_id,
-			    const GValue  *value,
-			    GParamSpec    *pspec)
+                            guint          prop_id,
+                            const GValue  *value,
+                            GParamSpec    *pspec)
 {
   PangoFcFont *fcfont = PANGO_FC_FONT (object);
 
@@ -219,25 +221,28 @@ pango_fc_font_set_property (GObject       *object,
     {
     case PROP_PATTERN:
       {
-	FcPattern *pattern = g_value_get_pointer (value);
+        FcPattern *pattern = g_value_get_pointer (value);
 
-	g_return_if_fail (pattern != NULL);
-	g_return_if_fail (fcfont->font_pattern == NULL);
+        g_return_if_fail (pattern != NULL);
+        g_return_if_fail (fcfont->font_pattern == NULL);
 
-	FcPatternReference (pattern);
-	fcfont->font_pattern = pattern;
-	fcfont->description = pango_fc_font_description_from_pattern (pattern, TRUE);
-	fcfont->is_hinted = pattern_is_hinted (pattern);
-	fcfont->is_transformed = pattern_is_transformed (pattern);
+        FcPatternReference (pattern);
+        fcfont->font_pattern = pattern;
+        fcfont->description = font_description_from_pattern (pattern, TRUE, TRUE);
+        fcfont->is_hinted = pattern_is_hinted (pattern);
+        fcfont->is_transformed = pattern_is_transformed (pattern);
       }
       goto set_decoder;
 
     case PROP_FONTMAP:
       {
-	PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (g_value_get_object (value));
+        PangoFcFontMap *fcfontmap = PANGO_FC_FONT_MAP (g_value_get_object (value));
 
-	g_return_if_fail (fcfont->fontmap == NULL);
-	g_weak_ref_set ((GWeakRef *) &fcfont->fontmap, fcfontmap);
+        g_return_if_fail (fcfont->fontmap == NULL);
+
+        fcfont->fontmap = (PangoFontMap *) fcfontmap;
+        if (fcfontmap)
+          g_object_add_weak_pointer (G_OBJECT (fcfontmap), (gpointer *) &fcfont->fontmap);
       }
       goto set_decoder;
 
@@ -271,8 +276,8 @@ pango_fc_font_get_property (GObject       *object,
     case PROP_FONTMAP:
       {
 	PangoFcFont *fcfont = PANGO_FC_FONT (object);
-	PangoFontMap *fontmap = g_weak_ref_get ((GWeakRef *) &fcfont->fontmap);
-	g_value_take_object (value, fontmap);
+
+	g_value_set_object (value, fcfont->fontmap);
       }
       break;
     default:
@@ -314,6 +319,13 @@ pango_fc_font_get_absolute_size (PangoFont *font)
   return 0;
 }
 
+static PangoVariant
+pango_fc_font_get_variant (PangoFont *font)
+{
+  PangoFcFont *fcfont = (PangoFcFont *)font;
+  return pango_font_description_get_variant (fcfont->description);
+}
+
 static PangoCoverage *
 pango_fc_font_get_coverage (PangoFont     *font,
 			    PangoLanguage *language G_GNUC_UNUSED)
@@ -321,8 +333,6 @@ pango_fc_font_get_coverage (PangoFont     *font,
   PangoFcFont *fcfont = (PangoFcFont *)font;
   PangoFcFontPrivate *priv = fcfont->priv;
   FcCharSet *charset;
-  PangoFcFontMap *fontmap;
-  PangoCoverage *coverage;
 
   if (priv->decoder)
     {
@@ -330,13 +340,10 @@ pango_fc_font_get_coverage (PangoFont     *font,
       return _pango_fc_font_map_fc_to_coverage (charset);
     }
 
-  fontmap = g_weak_ref_get ((GWeakRef *) &fcfont->fontmap);
-  if (!fontmap)
+  if (!fcfont->fontmap)
     return pango_coverage_new ();
 
-  coverage = _pango_fc_font_map_get_coverage (fontmap, fcfont);
-  g_object_unref (fontmap);
-  return coverage;
+  return _pango_fc_font_map_get_coverage (PANGO_FC_FONT_MAP (fcfont->fontmap), fcfont);
 }
 
 /* For Xft, it would be slightly more efficient to simply to
@@ -467,7 +474,7 @@ pango_fc_font_get_metrics (PangoFont     *font,
       PangoFontMap *fontmap;
       PangoContext *context;
 
-      fontmap = g_weak_ref_get ((GWeakRef *) &fcfont->fontmap);
+      fontmap = fcfont->fontmap;
       if (!fontmap)
 	return pango_font_metrics_new ();
 
@@ -516,7 +523,6 @@ pango_fc_font_get_metrics (PangoFont     *font,
         }
 
       g_object_unref (context);
-      g_object_unref (fontmap);
     }
 
   return pango_font_metrics_ref (info->metrics);
@@ -1081,8 +1087,8 @@ done:
  * The returned array is only valid as long as the font
  * and its fontmap are valid.
  *
- * Returns: (transfer none) (nullable): a %NULL-terminated
- *   array of `PangoLanguage`*
+ * Returns: (transfer none) (nullable) (array zero-terminated=1): a
+ *   %NULL-terminated array of `PangoLanguage`*
  *
  * Since: 1.48
  * Deprecated: 1.50: Use pango_font_get_language()
@@ -1097,17 +1103,11 @@ static PangoLanguage **
 _pango_fc_font_get_languages (PangoFont *font)
 {
   PangoFcFont * fcfont = PANGO_FC_FONT (font);
-  PangoFcFontMap *fontmap;
-  PangoLanguage **languages;
 
-  fontmap = g_weak_ref_get ((GWeakRef *) &fcfont->fontmap);
-  if (!fontmap)
+  if (!fcfont->fontmap)
     return NULL;
 
-  languages  = _pango_fc_font_map_get_languages (fontmap, fcfont);
-  g_object_unref (fontmap);
-
-  return languages;
+  return _pango_fc_font_map_get_languages (PANGO_FC_FONT_MAP (fcfont->fontmap), fcfont);
 }
 
 /**
