@@ -863,7 +863,6 @@ pango_win32_font_finalize (GObject *object)
 {
   PangoWin32Font *win32font = (PangoWin32Font *)object;
   PangoWin32FontCache *cache = pango_win32_font_map_get_font_cache (win32font->fontmap);
-  PangoWin32Font *fontmap;
 
   if (cache != NULL && win32font->hfont != NULL)
     pango_win32_font_cache_unload (cache, win32font->hfont);
@@ -876,12 +875,8 @@ pango_win32_font_finalize (GObject *object)
 
   g_hash_table_destroy (win32font->glyph_info);
 
-  fontmap = g_weak_ref_get ((GWeakRef *) &win32font->fontmap);
-  if (fontmap)
-  {
-    g_object_remove_weak_pointer (G_OBJECT (win32font->fontmap), (gpointer *) (gpointer) &win32font->fontmap);
-    g_object_unref (fontmap);
-  }
+  if (win32font->fontmap)
+    g_object_remove_weak_pointer (G_OBJECT (win32font->fontmap), (gpointer *) &win32font->fontmap);
 
   G_OBJECT_CLASS (_pango_win32_font_parent_class)->finalize (object);
 }
@@ -928,7 +923,8 @@ pango_win32_font_get_coverage (PangoFont     *font,
       while (hb_set_next(chars, &ch))
         pango_coverage_set (coverage, ch, PANGO_COVERAGE_EXACT);
 
-      win32face->coverage = g_object_ref (coverage);
+      hb_set_destroy (chars);
+      win32face->coverage = coverage;
     }
 
   return g_object_ref (win32face->coverage);
@@ -1319,14 +1315,12 @@ pango_win32_font_create_hb_font (PangoFont *font)
 #else
   hfont = _pango_win32_font_get_hfont (font);
 
-  /* We are *not* allowed to destroy the HFONT here ! */
 #ifdef USE_HB_GDI
   face = hb_gdi_face_create (hfont);
-
-  hb_face_set_user_data (face, &key, hfont, g_free, TRUE);
 #else
   face = hb_face_create_for_tables (hfont_reference_table, (void *)hfont, NULL);
 #endif
+
 #endif
 
   hb_font = hb_font_create (face);
